"use strict";
var extend = require("xtend");
var uniqueId = require("uniqueid");
var pascalCase = require("pascal-case");
var resource_1 = require("./resource");
function hasSecurity(api, type) {
    if (!api.securitySchemes)
        return false;
    return api.securitySchemes.some(function (schemes) {
        for (var _i = 0, _a = Object.keys(schemes); _i < _a.length; _i++) {
            var key = _a[_i];
            if (schemes[key].type === type) {
                return true;
            }
        }
        return false;
    });
}
exports.hasSecurity = hasSecurity;
function getSecuritySchemes(api) {
    var schemes = [];
    if (api.securitySchemes) {
        for (var _i = 0, _a = api.securitySchemes; _i < _a.length; _i++) {
            var nested = _a[_i];
            for (var _b = 0, _c = Object.keys(nested); _b < _c.length; _b++) {
                var key = _c[_b];
                schemes.push(nested[key]);
            }
        }
    }
    return schemes;
}
exports.getSecuritySchemes = getSecuritySchemes;
function allResources(api) {
    var array = [];
    function recurse(resources, prevUri, prevUriParams) {
        for (var _i = 0, resources_1 = resources; _i < resources_1.length; _i++) {
            var resource = resources_1[_i];
            var relativeUri = prevUri + resource.relativeUri;
            var uriParameters = resource_1.getUsedUriParameters(relativeUri, extend(resource.uriParameters, prevUriParams));
            var methods = resource.methods ? resource.methods : [];
            var description = resource.description;
            array.push({ methods: methods, relativeUri: relativeUri, uriParameters: uriParameters, description: description });
            if (resource.resources) {
                recurse(resource.resources, relativeUri, uriParameters);
            }
        }
    }
    recurse(api.resources, '', {});
    return array;
}
exports.allResources = allResources;
function nestedResources(api) {
    var methodId = uniqueId('Method');
    var resourceId = uniqueId('Resource');
    var keyUniqueness = {};
    var resource = {
        id: resourceId(),
        methodName: undefined,
        methods: [],
        relativeUri: '/',
        uriParameters: {},
        children: {}
    };
    function makeResource(node, child, segments) {
        if (segments.length === 0) {
            if (child.methods) {
                var _loop_1 = function (method) {
                    m = {
                        id: methodId(),
                        method: method.method,
                        queryParameters: method.queryParameters,
                        body: method.body,
                        headers: method.headers,
                        responseSchemas: {}
                    };
                    Object.keys(method.responses || {}).forEach(function (responseCode) {
                        var responseInfo = method.responses[responseCode].body || {};
                        m.responseSchemas[responseCode] = (responseInfo['application/json'] || {}).schema;
                    });
                    node.methods.push(m);
                };
                var m;
                for (var _i = 0, _a = child.methods; _i < _a.length; _i++) {
                    var method = _a[_i];
                    _loop_1(method);
                }
            }
            return node;
        }
        var key = segments[0];
        var childResource = key === '/' ? node : node.children[key];
        if (childResource == null) {
            var id = (node.id == "Resource0" ? "" : node.id) + pascalCase(key.replace(/^[a-zA-Z]/g, ""));
            if (keyUniqueness[id] != undefined) {
                id = keyUniqueness[id]();
            }
            else {
                keyUniqueness[id] = uniqueId(id);
            }
            childResource = node.children[key] = {
                id: id,
                methodName: resource_1.toMethodName(key),
                children: {},
                methods: [],
                uriParameters: resource_1.getUsedUriParameters(key, child.uriParameters),
                relativeUri: key
            };
        }
        return makeResource(childResource, child, segments.slice(1));
    }
    function handle(resource, children) {
        for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
            var child = children_1[_i];
            var segments = child.relativeUri.split(/(?=[\/\.])/g);
            var childResource = makeResource(resource, child, segments);
            if (child.resources) {
                handle(childResource, child.resources);
            }
        }
    }
    handle(resource, api.resources);
    return resource;
}
exports.nestedResources = nestedResources;
//# sourceMappingURL=api.js.map