"use strict";
var paramCase = require("param-case");
var pascalCase = require("pascal-case");
var camelCase = require("camel-case");
var strands_1 = require("strands");
var api_1 = require("../support/api");
var resource_1 = require("../support/resource");
var method_1 = require("../support/method");
function default_1(api) {
    var s = new strands_1.Strands();
    var projectName = paramCase(api.title);
    var className = pascalCase(api.title);
    var clientName = camelCase(api.title);
    s.multiline("# " + api.title + "\n\n> TypeScript abstraction library for RAML-based REST API [" + api.title + "](" + api.baseUri + ").\n\nAuto-generated using [raml-typescript-generator](https://github.com/brainboutique/raml-typescript-generator). \n\n## Installation\n\n```sh\nnpm install " + projectName + " --save\n```\n\n## Usage\n\n### TypeScript\n```ts\nimport {" + className + "} from '" + projectName + "';\n...\nconstructor(..) {\n  this." + clientName + " = new " + className + "({});\n}\n```\n\nTo support multiple versions of the API, it is recommended to alias the import so it can easily be mapped to a later API version - and, due to the nature of Typescript, \nyou should be alerted on API signature changes already at compile time:\n\n ```ts\nimport {" + className + " as " + className.replace(/V[0-9]*$/, "") + "} from '" + projectName + "';\n ```\n\n\n### JS (Legacy)\nAPI skeleton as it would be produced by MuleSoft's [raml-javascript-generator](https://github.com/mulesoft-labs/raml-javascript-generator) JS generator is shipped for reference and to ease migrations:\n```js\nvar " + className + " = require('" + projectName + "/leagcy.js')\n\nvar client = new " + className + "()\n```\n");
    if (api_1.hasSecurity(api, 'OAuth 2.0')) {
        s.multiline("### Authentication\n\n#### OAuth 2.0\n(!) Feature ported from legacy raml-javascript-generator but not yet finalized! You are on your own here...\n\nThis API supports authentication with [OAuth 2.0](https://github.com/mulesoft/js-client-oauth2). Initialize the `OAuth2` instance with the application client id, client secret and a redirect uri to authenticate with users.\n\n```js\nvar auth = new " + className + ".security.<method>({\n  clientId:     '123',\n  clientSecret: 'abc',\n  redirectUri:  'http://example.com/auth/callback'\n});\n\n// Available methods for OAuth 2.0:");
        for (var _i = 0, _a = api_1.getSecuritySchemes(api); _i < _a.length; _i++) {
            var scheme = _a[_i];
            if (scheme.type === 'OAuth 2.0') {
                s.line(" - " + camelCase(scheme.name));
            }
        }
        s.line('```');
    }
    s.multiline("### Options\n\nYou can set options when you initialize a client or at any time with the `options` property. You may also override options per request by passing an object as the last argument of request methods. For example:\n\n```javascript\nclient = new " + className + "({ ... })\n\nclient('GET', '/', {\n  baseUri: '" + (api.baseUri ? api.baseUri.replace(/\/[^/]*$/, "/anotherVersion") : "https://example.com") + "',\n  headers: {\n    'Content-Type': 'application/json'\n  }\n})\n```\n\nFor dynamic header injection - for example required for non-standard REST services asking for custom authentication token - a provider may be defined:\n\n```javascript\nclient = new " + className + "({\n  getHeaders: ()=>{ return(this.myToken ? {Authorization: \"Bearer \" + this.myToken} : {}) }\n});\n```\n\n#### Base URI\nBy default, endpoint as defined in RAML file `" + api.baseUri + "` is used.\n\n**Note** If supported by API provider, it is recommended to use one API version definition (i.e. RAML file and corresponding API TypeScript library) and switch endpoint based on the desired environment, for example `test`, `qa` or `prod`\n\nYou can override the base URI by setting the `baseUri` property, or initializing a client with a base URI. For example:\n\n```javascript\nnew " + className + "({\n  baseUri: '" + (api.baseUri ? api.baseUri.replace(/\/[^/]*$/, "/anotherVersion") : "https://example.com") + "',\n});\n```\n\n\n### Methods\n\nAll methods return an Observable wrapping a [Popsicle](https://github.com/blakeembrey/popsicle) obtained response:\n");
    for (var _b = 0, _c = api_1.allResources(api); _b < _c.length; _b++) {
        var resource = _c[_b];
        for (var _d = 0, _e = resource.methods; _d < _e.length; _d++) {
            var method = _e[_d];
            s.line("#### " + method_1.getDisplayName(method, resource));
            s.line();
            if (Object.keys(resource.uriParameters).length) {
                s.line(resource_1.getUriParametersSnippet(resource));
                s.line();
            }
            if (method.description) {
                s.multiline(method.description.trim());
                s.line();
            }
            s.multiline("```js\nclient." + method_1.getRequestSnippet(method, resource) + ".then(...)\n```\n  ");
        }
    }
    s.line('## License');
    s.line();
    s.line('Apache 2.0');
    return s.toString();
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
//# sourceMappingURL=README.md.js.map