"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var pascalCase = require("pascal-case");
var camelCase = require("camel-case");
var strands_1 = require("strands");
var stringify = require("javascript-stringify");
var parameters_1 = require("../../support/parameters");
var api_1 = require("../../support/api");
var method_1 = require("../../support/method");
var json_schema_to_typescript_1 = require("json-schema-to-typescript");
var clone = require('clone');
var IndentingStrands = (function (_super) {
    __extends(IndentingStrands, _super);
    function IndentingStrands() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.indentLevel = 0;
        _this.indentPattern = "  ";
        return _this;
    }
    IndentingStrands.prototype.setIndentLevel = function (i) {
        this.indentLevel = i;
    };
    IndentingStrands.prototype.setIndentPattern = function (s) {
        this.indentPattern = s;
    };
    IndentingStrands.prototype.line = function () {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        return _super.prototype.line.call(this, new Array(this.indentLevel + 1).join(this.indentPattern), values);
    };
    return IndentingStrands;
}(strands_1.Strands));
function default_1(api) {
    var s = new IndentingStrands();
    var className = pascalCase(api.title);
    var clientName = camelCase(api.title);
    s.line('/// <reference types="@types/es6-shim" />');
    s.line("//========================================================================");
    s.line("// RAML-defined Typed REST Client for '" + api.title + "'");
    s.line("//========================================================================");
    s.multiline("\nimport * as popsicle from 'popsicle';\nimport * as extend from 'deep-extend';\nimport {Observable} from '@reactivex/rxjs';\n\n");
    for (var _i = 0, _a = api.schemas; _i < _a.length; _i++) {
        var schemaWrapped = _a[_i];
        var schemaLogicalName = Object.keys(schemaWrapped)[0];
        var schema = JSON.parse(schemaWrapped[schemaLogicalName]);
        var schemaClassName = pascalCase(schemaLogicalName);
        s.line("//========================================================================");
        s.line("// Schema '" + schemaLogicalName + "'");
        s.line("//========================================================================");
        var options = {
            declareReferenced: true,
            endPropertyWithSemicolon: true,
            endTypeWithSemicolon: true,
            useConstEnums: true,
            useFullReferencePathAsName: false
        };
        var sanitizedSchema = clone(schema);
        sanitizedSchema.title = undefined;
        var typeDefinitions = json_schema_to_typescript_1.compile(sanitizedSchema, schemaClassName, options);
        s.line('export namespace ' + schemaClassName + " {");
        s.multiline(typeDefinitions.replace(/^(?!$)/mg, "   "));
        s.multiline("}\n\n");
    }
    var flatTree = api_1.allResources(api);
    var nestedTree = api_1.nestedResources(api);
    var _b = separateChildren(nestedTree), withParams = _b.withParams, noParams = _b.noParams;
    var supportedSecuritySchemes = api_1.getSecuritySchemes(api).filter(function (x) { return x.type === 'OAuth 2.0'; });
    if (api_1.hasSecurity(api, 'OAuth 2.0')) {
        s.line("import * as ClientOAuth2 from 'client-oauth2'");
    }
    s.line("//========================================================================");
    s.line("// Actions");
    s.line("//========================================================================");
    s.multiline("\n\nvar TEMPLATE_REGEXP = /{([^{}]+)}/g\n\nfunction template(string, interpolate) {\n  return string.replace(TEMPLATE_REGEXP, function (match, key) {\n    if (interpolate[key] != null) {\n      return encodeURIComponent(interpolate[key])\n    }\n\n    return ''\n  })\n}  \n\n export class " + className + " {\n  _client: any;\n  _path: string;\n  _options: any;\n  _form: any;\n  _version: any;\n  _Security: any;\n\n  // RAML resources without parameters\n");
    createThisResourcesNoParams(withParams, noParams, 'this', '', 'Resources.');
    s.multiline("\n  constructor(options?:any) {\n    this._path = ''\n    this._options = extend({\n      baseUri: " + stringify(api.baseUri) + ",\n      baseUriParameters: " + stringify(parameters_1.getDefaultParameters(api.baseUriParameters)) + "\n    }, options)\n  \n    //function client (method, path, options) {\n    //  return this.request(client, method, path, options)\n    //}\n    \n    // Initialize RAML resourcs without parameters\n");
    createThisResourcesNoParamsInitialization(withParams, noParams, 'this', '', 'Resources.');
    s.line("    this._client=this;");
    s.line("    this._form = popsicle.form");
    s.line("    this._version = " + stringify(api.version));
    s.line('    this._Security = {');
    supportedSecuritySchemes.forEach(function (scheme, index, schemes) {
        var name = camelCase(scheme.name);
        var trailing = index < schemes.length ? ',' : '';
        if (scheme.type === 'OAuth 2.0') {
            s.line("  " + name + ": function (options) { return new ClientOAuth2(extend(" + stringify(scheme.settings) + ", options)) }" + trailing);
        }
    });
    s.multiline("\n    }\n  // RAML resources with parameters\n  ");
    createThisResourcesWithParams(withParams, noParams, 'this', '', 'Resources.');
    s.multiline("\n}\n\n request (client, method, path, opts) {\n  var options = extend({}, client._options, opts)\n  var baseUri = template(options.baseUri, options.baseUriParameters)\n\n  var reqOpts = {\n    url: baseUri.replace(/\\/$/, '') + template(path, options.uriParameters),\n    method: method,\n    headers: extend(options.headers, options.getHeaders ? options.getHeaders() : {}),\n    body: options.body,\n    query: options.query,\n    options: options.options\n  }\n\n  if (options.user && typeof options.user.sign === 'function') {\n    reqOpts = options.user.sign(reqOpts)\n  }\n\n  return popsicle.request(reqOpts)\n}\n\n");
    for (var _c = 0, flatTree_1 = flatTree; _c < flatTree_1.length; _c++) {
        var resource = flatTree_1[_c];
        var relativeUri = resource.relativeUri, uriParameters = resource.uriParameters;
        for (var _d = 0, _e = resource.methods; _d < _e.length; _d++) {
            var method = _e[_d];
            if (method.annotations && method.annotations['client.methodName']) {
                var methodName = method.annotations['client.methodName'].structuredValue;
                var type = method_1.isQueryMethod(method) ? 'query' : 'body';
                var headers = parameters_1.getDefaultParameters(method.headers);
                if (Object.keys(uriParameters).length) {
                    s.line("export " + methodName + "(uriParams, " + type + ", opts) {");
                    s.line("  var uriParameters = extend(" + stringify(parameters_1.getDefaultParameters(uriParameters)) + ", uriParams)");
                    s.line("  var options = extend({ " + type + ": " + type + ", uriParameters: uriParameters, headers: " + stringify(headers) + " }, opts)");
                    s.line("  return this.request(this, " + stringify(method.method) + ", " + stringify(relativeUri) + ", options)");
                    s.line("}");
                }
                else {
                    s.line("export " + methodName + "(" + type + ", opts) {");
                    s.line("  var options = extend({ " + type + ": " + type + ", headers: " + stringify(headers) + " }, opts)");
                    s.line("  return this.request(this, " + stringify(method.method) + ", " + stringify(relativeUri) + ", options)");
                    s.line("}");
                }
            }
        }
    }
    createProtoResources(withParams, noParams, 'Client', 'Resources.');
    createProtoMethods(nestedTree.methods, 'Client', 'this', "''", 'Resources.');
    s.line("}");
    s.line("export module Resources {");
    createChildren(nestedTree.children, '');
    s.line("}");
    function createProtoMethods(methods, id, _client, _path, idPrefix) {
        var _loop_1 = function (method) {
            var headers = parameters_1.getDefaultParameters(method.headers);
            returnsJSON = false;
            schemasData = "{headers:any,responseCode:number, bodyRaw:any";
            Object.keys(method.responseSchemas).forEach(function (responseCode) {
                var schema = method.responseSchemas[responseCode];
                if (schema)
                    returnsJSON = true;
                if (schema && !schema.startsWith("{")) {
                    var schemaType = pascalCase(schema);
                    schemasData += ',"_' + responseCode + '"?:' + schemaType + "." + schemaType;
                }
                else if (schema) {
                    schemasData += ',"_' + responseCode + '":any  /* Implicitly defined schemas not yet supported */';
                }
                else
                    schemasData += ',"_' + responseCode + '":any';
            });
            schemasData += "}";
            queryParamsSignatureMandatory = "";
            queryParamsSignatureOptional = "";
            paramsInjection = [];
            if (method.queryParameters)
                Object.keys(method.queryParameters).forEach(function (parameterName) {
                    var param = method.queryParameters[parameterName];
                    var parameterJSName = camelCase(parameterName);
                    var tmp = parameterJSName;
                    if (!param.required)
                        tmp += "?";
                    switch (param.type) {
                        case "integer":
                            tmp += ":number";
                            break;
                        case "boolean":
                            tmp += ":" + param.type;
                            break;
                        default:
                            tmp += ":string";
                    }
                    if (!param.required)
                        queryParamsSignatureOptional += tmp + ", ";
                    else
                        queryParamsSignatureMandatory += tmp + ", ";
                    if (param.minimum)
                        paramsInjection.push("if (" + parameterJSName + " !== undefined && " + parameterJSName + " < " + param.minimum + ") { observer.error('Parameter \\'" + parameterJSName + "\\' outside specified range!'); return;}");
                    if (param.maximum)
                        paramsInjection.push("if (" + parameterJSName + " !== undefined && " + parameterJSName + " > " + param.maximum + ") { observer.error('Parameter \\'" + parameterJSName + "\\' outside specified range!'); return;}");
                    paramsInjection.push("if (" + parameterJSName + " !== undefined && " + parameterJSName + " !== null) options.query['" + parameterName + "']=" + parameterJSName + ";");
                });
            bodyParam = "";
            bodyParamInjection = [];
            if (!method_1.isQueryMethod(method) && method.body) {
                bodyParam += "body";
                if (method.body['application/json'] && method.body['application/json'].schema) {
                    schema = method.body['application/json'].schema;
                    if (!schema.startsWith("{")) {
                        schemaName = pascalCase(method.body['application/json'].schema);
                        bodyParam += ":" + schemaName + "." + schemaName + ", ";
                    }
                    else {
                        bodyParam += ":any /* Implicitly defined schemas not yet supported */, ";
                    }
                }
                else
                    bodyParam += ":any, ";
                paramsInjection.push("options.body=body;");
            }
            s.line("  /**");
            s.line("   * " + method.method.toUpperCase() + " on " + id);
            s.line("   */");
            s.line("  " + method.method.toUpperCase() + "(" + queryParamsSignatureMandatory + bodyParam + queryParamsSignatureOptional + "opts?:any):Observable<" + schemasData + "> {");
            s.line("    return Observable.create((observer) => {");
            s.line("       var options = extend({query:{}, headers: " + stringify(headers) + " }, opts)");
            paramsInjection.forEach(function (l) {
                s.line("       " + l);
            });
            s.line("       this._client.request(" + _client + ", " + stringify(method.method) + ", " + _path + ", options)");
            if (returnsJSON) {
                s.line("         .use(popsicle.plugins.parse('json'))");
            }
            s.line("         .then(response => {");
            s.line("             var r={headers:response.headers,responseCode:response.status,bodyRaw:response.body};");
            s.line("             r[\"_\"+response.status]=response.body; ");
            s.line("             observer.next(r);observer.complete()");
            s.line("      })");
            s.line("    })");
            s.line("  }");
        };
        var returnsJSON, schemasData, queryParamsSignatureMandatory, queryParamsSignatureOptional, paramsInjection, bodyParam, bodyParamInjection, schema, schemaName;
        for (var _i = 0, methods_1 = methods; _i < methods_1.length; _i++) {
            var method = methods_1[_i];
            _loop_1(method);
        }
    }
    function separateChildren(resource) {
        var withParams = {};
        var noParams = {};
        for (var _i = 0, _a = Object.keys(resource.children); _i < _a.length; _i++) {
            var key = _a[_i];
            var child = resource.children[key];
            if (Object.keys(child.uriParameters).length) {
                withParams[child.methodName] = child;
            }
            else {
                noParams[child.methodName] = child;
            }
        }
        return { withParams: withParams, noParams: noParams };
    }
    function toParamsFunction(child, _client, _prefix, idPrefix) {
        return "(uriParams:any) { return new " + idPrefix + child.id + "(" + _client + ", " + _prefix + "template(" + stringify(child.relativeUri) + ", extend(" + stringify(parameters_1.getDefaultParameters(child.uriParameters)) + ", uriParams))) }";
    }
    function toParamsFunctionSingle(parameterName, child, _client, _prefix, idPrefix) {
        return "(" + parameterName + ":string) { return new " + idPrefix + child.id + "(" + _client + ", " + _prefix + "template(" + stringify(child.relativeUri) + ", extend(" + stringify(parameters_1.getDefaultParameters(child.uriParameters)) + ", {" + parameterName + ":" + parameterName + "}))) }";
    }
    function createProtoResources(withParams, noParams, id, idPrefix) {
        for (var _i = 0, _a = Object.keys(withParams); _i < _a.length; _i++) {
            var key = _a[_i];
            var child = withParams[key];
            if (noParams[key] != null) {
                console.log("No params skip: " + key);
                continue;
            }
            s.line("// createProtoResources - Resource: " + id);
            s.line("" + child.methodName + toParamsFunctionSingle(key, child, 'this._client', 'this._path + ', idPrefix));
        }
    }
    function createResource(resource, idPrefix) {
        var _a = separateChildren(resource), withParams = _a.withParams, noParams = _a.noParams;
        s.line("// createResource - " + resource.id);
        s.line("export class " + resource.id + " { ");
        s.line("  _client: any; _path: string;");
        createThisResourcesNoParams(withParams, noParams, 'this._client', 'this._path + ', idPrefix);
        s.line("  constructor(client, path) {");
        s.line("    this._client = client");
        s.line("    this._path = path");
        createThisResourcesNoParamsInitialization(withParams, noParams, 'this._client', 'this._path + ', idPrefix);
        s.line("  }");
        createThisResourcesWithParams(withParams, noParams, 'this._client', 'this._path + ', idPrefix);
        createProtoResources(withParams, noParams, resource.id, idPrefix);
        createProtoMethods(resource.methods, resource.id, 'this._client', 'this._path', idPrefix);
        s.line("}");
        s.line("export module " + resource.id + " { ");
        createChildren(resource.children, idPrefix);
        s.line("}");
    }
    function createChildren(children, idPrefix) {
        var indentLevel = (idPrefix.match(/\./g) || []).length;
        ;
        s.setIndentLevel(indentLevel + 1);
        for (var _i = 0, _a = Object.keys(children); _i < _a.length; _i++) {
            var key = _a[_i];
            createResource(children[key], idPrefix + children[key].id + ".");
        }
        s.setIndentLevel(indentLevel);
    }
    function createThisResourcesWithParams(withParams, noParams, _client, _prefix, idPrefix) {
        for (var _i = 0, _a = Object.keys(noParams); _i < _a.length; _i++) {
            var key = _a[_i];
            var child = noParams[key];
            var constructor = "new " + idPrefix + child.id + "(" + _client + ", " + _prefix + stringify(child.relativeUri) + ")";
            if (withParams[key] == null) {
            }
            else {
                s.line("  // createThisResourcesWithParams - " + key);
                s.line("  " + child.methodName + "(" + toParamsFunction(withParams[key], _client, _prefix, idPrefix) + ", " + constructor + ")");
            }
        }
    }
    function createThisResourcesNoParams(withParams, noParams, _client, _prefix, idPrefix) {
        for (var _i = 0, _a = Object.keys(noParams); _i < _a.length; _i++) {
            var key = _a[_i];
            var child = noParams[key];
            if (withParams[key] == null) {
                s.line("  " + child.methodName + ":" + idPrefix + child.id + ";");
            }
        }
    }
    function createThisResourcesNoParamsInitialization(withParams, noParams, _client, _prefix, idPrefix) {
        for (var _i = 0, _a = Object.keys(noParams); _i < _a.length; _i++) {
            var key = _a[_i];
            var child = noParams[key];
            var constructor = "new " + idPrefix + child.id + "(" + _client + ", " + _prefix + stringify(child.relativeUri) + ")";
            if (withParams[key] == null) {
                s.line("  this." + child.methodName + " = " + constructor);
            }
        }
    }
    return s.toString();
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
//# sourceMappingURL=index.ts.js.map